// EnumProcesses.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "console.h"

#pragma pack(16)

typedef struct tagPROCESSTIMES
{
	__int64 m_kernel;
	__int64 m_user;
} PROCESSTIMES[100];

PROCESSTIMES ptm;

#define WAIT_FOR_PROCESS 5	// In seconds

// GetCurrentProcessCreateTime - retrieves the create time and converts the
//                   time to a string
// Return value - TRUE if successful, FALSE otherwise
// lpszString   - pointer to buffer to receive string

BOOL GetCurrentProcessCreateTime(LPSTR lpszString)
{
    // for this process
    FILETIME createTime;
    FILETIME exitTime;
    FILETIME kernelTimeProcess;
    FILETIME userTimeProcess;
	FILETIME ftLocal;
	HANDLE hProcess = GetCurrentProcess();
    SYSTEMTIME stCreate;
	
    // Retrieve the file times for the process.
	if (hProcess)
	{
		if (!GetProcessTimes(hProcess, &createTime, &exitTime, &kernelTimeProcess, &userTimeProcess))
			return FALSE;
		
		// Convert the last-write time to local time.
		if (!FileTimeToLocalFileTime(&createTime, &ftLocal))
			return FALSE;
		
		// Convert the local file time from UTC to system time.
		FileTimeToSystemTime(&ftLocal, &stCreate);
		
		// Build a string showing the date and time.
		wsprintf(lpszString, "%02d/%02d/%d %02d:%02d:%02d",
			stCreate.wMonth, stCreate.wDay, stCreate.wYear,
			stCreate.wHour, stCreate.wMinute, stCreate.wSecond);
		
		return TRUE;
	}
	return FALSE;
}

// GetTimes     - retrieves the current values of kernel time and user time
//				  and saves them in structure PROCESSTIMES[index] as
//				  64 bit values.
// processID    - the process identifier (PID) of a process.
// index	    - the index used to store data in the structure array.
// Return value - this function does not return a value.
//
void GetTimes( DWORD processID, UINT index)
{
	// for the first process sample
	FILETIME m_createTime;
	FILETIME m_exitTime;
	FILETIME m_kernelTimeProcess;
	FILETIME m_userTimeProcess;
	__int64 m_kernel = 0;
	__int64 m_user = 0;

    // Get a handle to the process.
    HANDLE hProcess = OpenProcess( PROCESS_QUERY_INFORMATION |
                                   PROCESS_VM_READ,
                                   FALSE, processID );
    if ( hProcess )
	{
	    GetProcessTimes( hProcess, &m_createTime, &m_exitTime, &m_kernelTimeProcess, &m_userTimeProcess );
		m_kernel = m_kernelTimeProcess.dwHighDateTime;
		m_kernel = m_kernel << 32;
		m_kernel += m_kernelTimeProcess.dwLowDateTime;
		ptm[index].m_kernel = m_kernel;
		m_user = m_userTimeProcess.dwHighDateTime;
		m_user = m_user << 32;
		m_user += m_userTimeProcess.dwLowDateTime;
		ptm[index].m_user = m_user;
	}
}

// PrintProcessNameAndID - retrieves the current values of kernel time and user time
//				  and compares them to the saved values in structure PROCESSTIMES[index]
//				  they are then printed to the screen.
// processID    - the process identifier (PID) of a process.
// index	    - the index used to retreive stored data in the structure array.
// Return value - cpuusage, the sum of active process times.
//
// Comment:		  MSDN Library (October 2001) documentation of GetProccessTimes states:
//      		  Process kernel mode and user mode times are amounts of time.
//				  For example, if a process has spent one second in kernel mode,
//				  this function will fill the FILETIME structure specified by
//				  lpKernelTime with a 64-bit value of ten million.
//				  That is the number of 100-nanosecond units in one second. 
//
// Note:		  I found this value to be 156250 as a FILETIME value on my system.
//
ULONG PrintProcessNameAndID( DWORD processID, UINT index )
{
    char szProcessName[MAX_PATH] = "unknown";

    // Get a handle to the process.

    HANDLE hProcess = OpenProcess( PROCESS_QUERY_INFORMATION |
                                   PROCESS_VM_READ,
                                   FALSE, processID );

    // Get the process name.

    if ( hProcess )
    {
        HMODULE hMod;
        DWORD cbNeeded;

        if ( EnumProcessModules( hProcess, &hMod, sizeof(hMod), 
             &cbNeeded) )
        {
            GetModuleBaseName( hProcess, hMod, szProcessName, 
                               sizeof(szProcessName) );
        }
    }

    // for this process
    FILETIME createTime;
    FILETIME exitTime;
    FILETIME kernelTimeProcess;
    FILETIME userTimeProcess;
	__int64 kernel = 0;
	__int64 user = 0;
	ULONG kresult = 0;
	ULONG uresult = 0;
	ULONG cpuusage = 0;
	__int64 divisor = 156250;

    GetProcessTimes( hProcess, &createTime, &exitTime, &kernelTimeProcess, &userTimeProcess );
    // Print the process name and identifier.
	kernel = kernelTimeProcess.dwHighDateTime;
	kernel = kernel << 32;
	kernel += kernelTimeProcess.dwLowDateTime;
 	kernel -= ptm[index].m_kernel;
	kresult = (ULONG)(kernel / divisor);
	user = userTimeProcess.dwHighDateTime;
	user = user << 32;
	user += userTimeProcess.dwLowDateTime;
	user -= ptm[index].m_user;
	uresult = (ULONG)(user / divisor);
	if(strcmp(szProcessName, "unknown") != 0)
	{
		if((kresult != 0) || (uresult != 0))
			setrgb(BLACK, INT_YELLOW);
	    printf( "%s", szProcessName );
		setrgb(BLACK, INT_WHITE);
		if(strlen(szProcessName) < 8)
		    printf( "\t\t\t(PID: %u)", processID );
		else if(strlen(szProcessName) > 16)
		    printf( "\t(PID: %u)", processID );
		else
		    printf( "\t\t(PID: %u)", processID );
 		printf( "\tKernel [");
		if(kresult != 0)
		{
			setrgb(BLACK, INT_YELLOW);
			printf( "%lu", kresult);
			cpuusage += kresult;
			setrgb(BLACK, INT_WHITE);
		}
		else
		{
			setrgb(BLACK, INT_WHITE);
			printf( "%lu", kresult);
		}
		printf( "]\t User [");
		if(uresult != 0)
		{
			setrgb(BLACK, INT_YELLOW);
			printf( "%lu", uresult);
			cpuusage += uresult;
			setrgb(BLACK, INT_WHITE);
		}
		else
		{
			setrgb(BLACK, INT_WHITE);
			printf( "%lu", uresult);
		}
		printf( "]\n");
	}
    CloseHandle( hProcess );
	return cpuusage;
}

void main( )
{
	setrgb(BLACK, INT_WHITE);
	clrscr(MINIMUM);
	char String[32];
	LPSTR lpszString;
	lpszString = String;
	if(GetCurrentProcessCreateTime(lpszString))
	{
		gotoxy(20,10);
		printf("This Process Created: %s\n", lpszString);
	}
	setrgb(BLACK, INT_RED);
	gotoxy(26,12);
	printf("Processing, PLEASE wait");
	setrgb(BLACK, INT_WHITE);
    // Get the list of process identifiers.
    DWORD aProcesses[1024], cbNeeded, cProcesses;
	ULONG lCpuusage = 0;
    UINT index;
	double percent;
    if ( !EnumProcesses( aProcesses, sizeof(aProcesses), &cbNeeded ) )
        return;

    // Calculate how many process identifiers were returned.
    cProcesses = cbNeeded / sizeof(DWORD);
	if(cProcesses > 100)
	{
		printf("\nProgram Error, %ld is too many, see PROCESSTIMES  structure!\n", cProcesses);
		printf("\nPress any key to exit ");
		while(!kbhit());
		exit(1);
	}

	// Get process times
    for ( index = 0; index < cProcesses; index++ )
	{
        GetTimes( aProcesses[index], index );
	}
	//Wait 5 seconds
	for(int wait = 0; wait < WAIT_FOR_PROCESS; wait++)
	{
		printf(".");
		Sleep(1000);
	}
	clrscr(MAXIMUM);
	setrgb(BLACK, INT_WHITE);
	printf("Process Name           |    PID    |  | Kernel    |    | User    |\n");
	printf("------------------------------------------------------------------\n");
    // Print the name and process identifier for each process.
    for ( index = 0; index < cProcesses; index++ )
	{
        lCpuusage += PrintProcessNameAndID( aProcesses[index], index );
	}
	percent = (double)(lCpuusage / (double)WAIT_FOR_PROCESS);
	printf("------------------------------------------------------------------\n");
	printf("Processes: %ld\t\t CPU Usage: ", cProcesses);
	setrgb(BLACK, INT_YELLOW);
	printf("%3.2f", percent);
	setrgb(BLACK, INT_WHITE);
	printf("%% Per/Sec\n");
	printf("\nPress any key to exit ");
	while(!kbhit());
}

